@@ -1,4 +1,4 @@
-// +build !windows
+//go:build !windows
 
 /*
    Copyright The containerd Authors.
@@ -22,22 +22,23 @@ import (
 	"archive/tar"
 	"bytes"
 	"context"
+	_ "crypto/sha256"
+	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
 	"os/exec"
 	"path/filepath"
+	"runtime"
 	"testing"
 	"time"
 
-	_ "crypto/sha256"
-
 	"github.com/containerd/containerd/archive/tartest"
 	"github.com/containerd/containerd/pkg/testutil"
 	"github.com/containerd/continuity/fs"
 	"github.com/containerd/continuity/fs/fstest"
-	"github.com/pkg/errors"
+	"github.com/opencontainers/go-digest"
+	"github.com/stretchr/testify/require"
 )
 
 const tarCmd = "tar"
@@ -58,7 +59,7 @@ var baseApplier = fstest.Apply(
 func TestUnpack(t *testing.T) {
 	requireTar(t)
 
-	if err := testApply(baseApplier); err != nil {
+	if err := testApply(t, baseApplier); err != nil {
 		t.Fatalf("Test apply failed: %+v", err)
 	}
 }
@@ -66,7 +67,7 @@ func TestUnpack(t *testing.T) {
 func TestBaseDiff(t *testing.T) {
 	requireTar(t)
 
-	if err := testBaseDiff(baseApplier); err != nil {
+	if err := testBaseDiff(t, baseApplier); err != nil {
 		t.Fatalf("Test base diff failed: %+v", err)
 	}
 }
@@ -102,7 +103,7 @@ func TestRelativeSymlinks(t *testing.T) {
 	}
 
 	for _, bo := range breakoutLinks {
-		if err := testDiffApply(bo); err != nil {
+		if err := testDiffApply(t, bo); err != nil {
 			t.Fatalf("Test apply failed: %+v", err)
 		}
 	}
@@ -179,7 +180,7 @@ func TestSymlinks(t *testing.T) {
 	}
 
 	for i, l := range links {
-		if err := testDiffApply(l[0], l[1]); err != nil {
+		if err := testDiffApply(t, l[0], l[1]); err != nil {
 			t.Fatalf("Test[%d] apply failed: %+v", i+1, err)
 		}
 	}
@@ -233,21 +234,17 @@ func TestBreakouts(t *testing.T) {
 	tc := tartest.TarContext{}.WithUIDGID(os.Getuid(), os.Getgid()).WithModTime(time.Now().UTC())
 	expected := "unbroken"
 	unbrokenCheck := func(root string) error {
-		b, err := ioutil.ReadFile(filepath.Join(root, "etc", "unbroken"))
+		b, err := os.ReadFile(filepath.Join(root, "etc", "unbroken"))
 		if err != nil {
-			return errors.Wrap(err, "failed to read unbroken")
+			return fmt.Errorf("failed to read unbroken: %w", err)
 		}
 		if string(b) != expected {
-			return errors.Errorf("/etc/unbroken: unexpected value %s, expected %s", b, expected)
+			return fmt.Errorf("/etc/unbroken: unexpected value %s, expected %s", b, expected)
 		}
 		return nil
 	}
 	errFileDiff := errors.New("files differ")
-	td, err := ioutil.TempDir("", "test-breakouts-")
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer os.RemoveAll(td)
+	td := t.TempDir()
 
 	isSymlinkFile := func(f string) func(string) error {
 		return func(root string) error {
@@ -257,7 +254,7 @@ func TestBreakouts(t *testing.T) {
 			}
 
 			if got := fi.Mode() & os.ModeSymlink; got != os.ModeSymlink {
-				return errors.Errorf("%s should be symlink", fi.Name())
+				return fmt.Errorf("%s should be symlink", fi.Name())
 			}
 			return nil
 		}
@@ -285,7 +282,7 @@ func TestBreakouts(t *testing.T) {
 			}
 
 			if t1 != t2 {
-				return errors.Wrapf(errFileDiff, "%#v and %#v", t1, t2)
+				return fmt.Errorf("%#v and %#v: %w", t1, t2, errFileDiff)
 			}
 			return nil
 		}
@@ -310,7 +307,7 @@ func TestBreakouts(t *testing.T) {
 				return err
 			}
 			if !os.SameFile(s1, s2) {
-				return errors.Wrapf(errFileDiff, "%#v and %#v", s1, s2)
+				return fmt.Errorf("%#v and %#v: %w", s1, s2, errFileDiff)
 			}
 			return nil
 		}
@@ -330,12 +327,12 @@ func TestBreakouts(t *testing.T) {
 	}
 	fileValue := func(f1 string, content []byte) func(string) error {
 		return func(root string) error {
-			b, err := ioutil.ReadFile(filepath.Join(root, f1))
+			b, err := os.ReadFile(filepath.Join(root, f1))
 			if err != nil {
 				return err
 			}
 			if !bytes.Equal(b, content) {
-				return errors.Errorf("content differs: expected %v, got %v", content, b)
+				return fmt.Errorf("content differs: expected %v, got %v", content, b)
 			}
 			return nil
 		}
@@ -363,13 +360,14 @@ func TestBreakouts(t *testing.T) {
 		}
 	}
 
-	breakouts := []struct {
+	type breakoutTest struct {
 		name      string
 		w         tartest.WriterToTar
 		apply     fstest.Applier
 		validator func(string) error
 		err       error
-	}{
+	}
+	breakouts := []breakoutTest{
 		{
 			name: "SymlinkAbsolute",
 			w: tartest.TarAll(
@@ -420,12 +418,12 @@ func TestBreakouts(t *testing.T) {
 				tc.File("/localetc/emptied", []byte{}, 0644),
 			),
 			validator: func(root string) error {
-				b, err := ioutil.ReadFile(filepath.Join(root, "etc", "emptied"))
+				b, err := os.ReadFile(filepath.Join(root, "etc", "emptied"))
 				if err != nil {
-					return errors.Wrap(err, "failed to read unbroken")
+					return fmt.Errorf("failed to read unbroken: %w", err)
 				}
 				if len(b) > 0 {
-					return errors.Errorf("/etc/emptied: non-empty")
+					return errors.New("/etc/emptied: non-empty")
 				}
 				return nil
 			},
@@ -501,42 +499,7 @@ func TestBreakouts(t *testing.T) {
 			),
 			validator: sameFile("localpasswd", "/etc/passwd"),
 		},
-		{
-			name: "HardlinkSymlinkBeforeCreateTarget",
-			w: tartest.TarAll(
-				tc.Dir("etc", 0770),
-				tc.Symlink("/etc/passwd", "localpasswd"),
-				tc.Link("localpasswd", "localpasswd-dup"),
-				tc.File("/etc/passwd", []byte("after"), 0644),
-			),
-			validator: sameFile("localpasswd-dup", "/etc/passwd"),
-		},
-		{
-			name: "HardlinkSymlinkRelative",
-			w: tartest.TarAll(
-				tc.Dir("etc", 0770),
-				tc.File("/etc/passwd", []byte("inside"), 0644),
-				tc.Symlink("../../../../../etc/passwd", "passwdlink"),
-				tc.Link("/passwdlink", "localpasswd"),
-			),
-			validator: all(
-				sameSymlinkFile("/localpasswd", "/passwdlink"),
-				sameFile("/localpasswd", "/etc/passwd"),
-			),
-		},
-		{
-			name: "HardlinkSymlinkAbsolute",
-			w: tartest.TarAll(
-				tc.Dir("etc", 0770),
-				tc.File("/etc/passwd", []byte("inside"), 0644),
-				tc.Symlink("/etc/passwd", "passwdlink"),
-				tc.Link("/passwdlink", "localpasswd"),
-			),
-			validator: all(
-				sameSymlinkFile("/localpasswd", "/passwdlink"),
-				sameFile("/localpasswd", "/etc/passwd"),
-			),
-		},
+
 		{
 			name: "SymlinkParentDirectory",
 			w: tartest.TarAll(
@@ -749,36 +712,77 @@ func TestBreakouts(t *testing.T) {
 			// resolution ends up just removing etc
 			validator: fileNotExists("etc/passwd"),
 		},
-		{
+	}
 
-			name: "HardlinkSymlinkChmod",
-			w: func() tartest.WriterToTar {
-				p := filepath.Join(td, "perm400")
-				if err := ioutil.WriteFile(p, []byte("..."), 0400); err != nil {
-					t.Fatal(err)
-				}
-				ep := filepath.Join(td, "also-exists-outside-root")
-				if err := ioutil.WriteFile(ep, []byte("..."), 0640); err != nil {
-					t.Fatal(err)
-				}
-
-				return tartest.TarAll(
-					tc.Symlink(p, ep),
-					tc.Link(ep, "sketchylink"),
-				)
-			}(),
-			validator: func(string) error {
-				p := filepath.Join(td, "perm400")
-				fi, err := os.Lstat(p)
-				if err != nil {
-					return err
-				}
-				if perm := fi.Mode() & os.ModePerm; perm != 0400 {
-					return errors.Errorf("%s perm changed from 0400 to %04o", p, perm)
-				}
-				return nil
+	// The follow tests perform operations not permitted on Darwin
+	if runtime.GOOS != "darwin" {
+		breakouts = append(breakouts, []breakoutTest{
+			{
+				name: "HardlinkSymlinkBeforeCreateTarget",
+				w: tartest.TarAll(
+					tc.Dir("etc", 0770),
+					tc.Symlink("/etc/passwd", "localpasswd"),
+					tc.Link("localpasswd", "localpasswd-dup"),
+					tc.File("/etc/passwd", []byte("after"), 0644),
+				),
+				validator: sameFile("localpasswd-dup", "/etc/passwd"),
 			},
-		},
+			{
+				name: "HardlinkSymlinkRelative",
+				w: tartest.TarAll(
+					tc.Dir("etc", 0770),
+					tc.File("/etc/passwd", []byte("inside"), 0644),
+					tc.Symlink("../../../../../etc/passwd", "passwdlink"),
+					tc.Link("/passwdlink", "localpasswd"),
+				),
+				validator: all(
+					sameSymlinkFile("/localpasswd", "/passwdlink"),
+					sameFile("/localpasswd", "/etc/passwd"),
+				),
+			},
+			{
+				name: "HardlinkSymlinkAbsolute",
+				w: tartest.TarAll(
+					tc.Dir("etc", 0770),
+					tc.File("/etc/passwd", []byte("inside"), 0644),
+					tc.Symlink("/etc/passwd", "passwdlink"),
+					tc.Link("/passwdlink", "localpasswd"),
+				),
+				validator: all(
+					sameSymlinkFile("/localpasswd", "/passwdlink"),
+					sameFile("/localpasswd", "/etc/passwd"),
+				),
+			},
+			{
+				name: "HardlinkSymlinkChmod",
+				w: func() tartest.WriterToTar {
+					p := filepath.Join(td, "perm400")
+					if err := os.WriteFile(p, []byte("..."), 0400); err != nil {
+						t.Fatal(err)
+					}
+					ep := filepath.Join(td, "also-exists-outside-root")
+					if err := os.WriteFile(ep, []byte("..."), 0640); err != nil {
+						t.Fatal(err)
+					}
+
+					return tartest.TarAll(
+						tc.Symlink(p, ep),
+						tc.Link(ep, "sketchylink"),
+					)
+				}(),
+				validator: func(string) error {
+					p := filepath.Join(td, "perm400")
+					fi, err := os.Lstat(p)
+					if err != nil {
+						return err
+					}
+					if perm := fi.Mode() & os.ModePerm; perm != 0400 {
+						return fmt.Errorf("%s perm changed from 0400 to %04o", p, perm)
+					}
+					return nil
+				},
+			},
+		}...)
 	}
 
 	for _, bo := range breakouts {
@@ -800,7 +804,7 @@ func TestApplyTar(t *testing.T) {
 					return err
 				}
 				if _, err := os.Stat(p); err != nil {
-					return errors.Wrapf(err, "failure checking existence for %v", d)
+					return fmt.Errorf("failure checking existence for %v: %w", d, err)
 				}
 			}
 			return nil
@@ -841,26 +845,18 @@ func TestApplyTar(t *testing.T) {
 	}
 }
 
-func testApply(a fstest.Applier) error {
-	td, err := ioutil.TempDir("", "test-apply-")
-	if err != nil {
-		return errors.Wrap(err, "failed to create temp dir")
-	}
-	defer os.RemoveAll(td)
-	dest, err := ioutil.TempDir("", "test-apply-dest-")
-	if err != nil {
-		return errors.Wrap(err, "failed to create temp dir")
-	}
-	defer os.RemoveAll(dest)
+func testApply(t *testing.T, a fstest.Applier) error {
+	td := t.TempDir()
+	dest := t.TempDir()
 
 	if err := a.Apply(td); err != nil {
-		return errors.Wrap(err, "failed to apply filesystem changes")
+		return fmt.Errorf("failed to apply filesystem changes: %w", err)
 	}
 
-	tarArgs := []string{"c", "-C", td}
+	tarArgs := []string{"cf", "-", "-C", td}
 	names, err := readDirNames(td)
 	if err != nil {
-		return errors.Wrap(err, "failed to read directory names")
+		return fmt.Errorf("failed to read directory names: %w", err)
 	}
 	tarArgs = append(tarArgs, names...)
 
@@ -868,62 +864,49 @@ func testApply(a fstest.Applier) error {
 
 	arch, err := cmd.StdoutPipe()
 	if err != nil {
-		return errors.Wrap(err, "failed to create stdout pipe")
+		return fmt.Errorf("failed to create stdout pipe: %w", err)
 	}
 
 	if err := cmd.Start(); err != nil {
-		return errors.Wrap(err, "failed to start command")
+		return fmt.Errorf("failed to start command: %w", err)
 	}
 
 	if _, err := Apply(context.Background(), dest, arch); err != nil {
-		return errors.Wrap(err, "failed to apply tar stream")
+		return fmt.Errorf("failed to apply tar stream: %w", err)
 	}
 
 	return fstest.CheckDirectoryEqual(td, dest)
 }
 
-func testBaseDiff(a fstest.Applier) error {
-	td, err := ioutil.TempDir("", "test-base-diff-")
-	if err != nil {
-		return errors.Wrap(err, "failed to create temp dir")
-	}
-	defer os.RemoveAll(td)
-	dest, err := ioutil.TempDir("", "test-base-diff-dest-")
-	if err != nil {
-		return errors.Wrap(err, "failed to create temp dir")
-	}
-	defer os.RemoveAll(dest)
+func testBaseDiff(t *testing.T, a fstest.Applier) error {
+	td := t.TempDir()
+	dest := t.TempDir()
 
 	if err := a.Apply(td); err != nil {
-		return errors.Wrap(err, "failed to apply filesystem changes")
+		return fmt.Errorf("failed to apply filesystem changes: %w", err)
 	}
 
 	arch := Diff(context.Background(), "", td)
 
-	cmd := exec.Command(tarCmd, "x", "-C", dest)
+	cmd := exec.Command(tarCmd, "xf", "-", "-C", dest)
 	cmd.Stdin = arch
+	stderr := &bytes.Buffer{}
+	cmd.Stderr = stderr
 	if err := cmd.Run(); err != nil {
-		return errors.Wrap(err, "tar command failed")
+		fmt.Println(stderr.String())
+		return fmt.Errorf("tar command failed: %w", err)
 	}
 
 	return fstest.CheckDirectoryEqual(td, dest)
 }
 
-func testDiffApply(appliers ...fstest.Applier) error {
-	td, err := ioutil.TempDir("", "test-diff-apply-")
-	if err != nil {
-		return errors.Wrap(err, "failed to create temp dir")
-	}
-	defer os.RemoveAll(td)
-	dest, err := ioutil.TempDir("", "test-diff-apply-dest-")
-	if err != nil {
-		return errors.Wrap(err, "failed to create temp dir")
-	}
-	defer os.RemoveAll(dest)
+func testDiffApply(t *testing.T, appliers ...fstest.Applier) error {
+	td := t.TempDir()
+	dest := t.TempDir()
 
 	for _, a := range appliers {
 		if err := a.Apply(td); err != nil {
-			return errors.Wrap(err, "failed to apply filesystem changes")
+			return fmt.Errorf("failed to apply filesystem changes: %w", err)
 		}
 	}
 
@@ -931,18 +914,18 @@ func testDiffApply(appliers ...fstest.Applier) error {
 	if len(appliers) > 1 {
 		for _, a := range appliers[:len(appliers)-1] {
 			if err := a.Apply(dest); err != nil {
-				return errors.Wrap(err, "failed to apply base filesystem changes")
+				return fmt.Errorf("failed to apply base filesystem changes: %w", err)
 			}
 		}
 	}
 
-	diffBytes, err := ioutil.ReadAll(Diff(context.Background(), dest, td))
+	diffBytes, err := io.ReadAll(Diff(context.Background(), dest, td))
 	if err != nil {
-		return errors.Wrap(err, "failed to create diff")
+		return fmt.Errorf("failed to create diff: %w", err)
 	}
 
 	if _, err := Apply(context.Background(), dest, bytes.NewReader(diffBytes)); err != nil {
-		return errors.Wrap(err, "failed to apply tar stream")
+		return fmt.Errorf("failed to apply tar stream: %w", err)
 	}
 
 	return fstest.CheckDirectoryEqual(td, dest)
@@ -950,11 +933,7 @@ func testDiffApply(appliers ...fstest.Applier) error {
 
 func makeWriterToTarTest(wt tartest.WriterToTar, a fstest.Applier, validate func(string) error, applyErr error) func(*testing.T) {
 	return func(t *testing.T) {
-		td, err := ioutil.TempDir("", "test-writer-to-tar-")
-		if err != nil {
-			t.Fatalf("Failed to create temp dir: %v", err)
-		}
-		defer os.RemoveAll(td)
+		td := t.TempDir()
 
 		if a != nil {
 			if err := a.Apply(td); err != nil {
@@ -1186,18 +1165,64 @@ func TestDiffTar(t *testing.T) {
 	}
 }
 
+func TestSourceDateEpoch(t *testing.T) {
+	sourceDateEpoch, err := time.Parse(time.RFC3339, "2022-01-23T12:34:56Z")
+	require.NoError(t, err)
+	past, err := time.Parse(time.RFC3339, "2022-01-01T00:00:00Z")
+	require.NoError(t, err)
+	require.True(t, past.Before(sourceDateEpoch))
+	veryRecent := time.Now()
+	require.True(t, veryRecent.After(sourceDateEpoch))
+
+	opts := []WriteDiffOpt{WithSourceDateEpoch(&sourceDateEpoch)}
+	validators := []tarEntryValidator{
+		composeValidators(whiteoutEntry("f1"), requireModTime(sourceDateEpoch)),
+		composeValidators(fileEntry("f2", []byte("content2"), 0644), requireModTime(past)),
+		composeValidators(fileEntry("f3", []byte("content3"), 0644), requireModTime(sourceDateEpoch)),
+	}
+	a := fstest.Apply(
+		fstest.CreateFile("/f1", []byte("content"), 0644),
+	)
+	b := fstest.Apply(
+		// Remove f1; the timestamp of the tar entry will be sourceDateEpoch
+		fstest.RemoveAll("/f1"),
+		// Create f2 with the past timestamp; the timestamp of the tar entry will be past (< sourceDateEpoch)
+		fstest.CreateFile("/f2", []byte("content2"), 0644),
+		fstest.Chtimes("/f2", past, past),
+		// Create f3 with the veryRecent timestamp; the timestamp of the tar entry will be sourceDateEpoch
+		fstest.CreateFile("/f3", []byte("content3"), 0644),
+		fstest.Chtimes("/f3", veryRecent, veryRecent),
+	)
+	makeDiffTarTest(validators, a, b, opts...)(t)
+	if testing.Short() {
+		t.Skip("short: skipping repro test")
+	}
+	makeDiffTarReproTest(a, b, opts...)(t)
+}
+
 type tarEntryValidator func(*tar.Header, []byte) error
 
+func composeValidators(vv ...tarEntryValidator) tarEntryValidator {
+	return func(hdr *tar.Header, b []byte) error {
+		for _, v := range vv {
+			if err := v(hdr, b); err != nil {
+				return err
+			}
+		}
+		return nil
+	}
+}
+
 func dirEntry(name string, mode int) tarEntryValidator {
 	return func(hdr *tar.Header, b []byte) error {
 		if hdr.Typeflag != tar.TypeDir {
 			return errors.New("not directory type")
 		}
 		if hdr.Name != name {
-			return errors.Errorf("wrong name %q, expected %q", hdr.Name, name)
+			return fmt.Errorf("wrong name %q, expected %q", hdr.Name, name)
 		}
 		if hdr.Mode != int64(mode) {
-			return errors.Errorf("wrong mode %o, expected %o", hdr.Mode, mode)
+			return fmt.Errorf("wrong mode %o, expected %o", hdr.Mode, mode)
 		}
 		return nil
 	}
@@ -1209,13 +1234,13 @@ func fileEntry(name string, expected []byte, mode int) tarEntryValidator {
 			return errors.New("not file type")
 		}
 		if hdr.Name != name {
-			return errors.Errorf("wrong name %q, expected %q", hdr.Name, name)
+			return fmt.Errorf("wrong name %q, expected %q", hdr.Name, name)
 		}
 		if hdr.Mode != int64(mode) {
-			return errors.Errorf("wrong mode %o, expected %o", hdr.Mode, mode)
+			return fmt.Errorf("wrong mode %o, expected %o", hdr.Mode, mode)
 		}
 		if !bytes.Equal(b, expected) {
-			return errors.Errorf("different file content")
+			return errors.New("different file content")
 		}
 		return nil
 	}
@@ -1227,10 +1252,10 @@ func linkEntry(name, link string) tarEntryValidator {
 			return errors.New("not link type")
 		}
 		if hdr.Name != name {
-			return errors.Errorf("wrong name %q, expected %q", hdr.Name, name)
+			return fmt.Errorf("wrong name %q, expected %q", hdr.Name, name)
 		}
 		if hdr.Linkname != link {
-			return errors.Errorf("wrong link %q, expected %q", hdr.Linkname, link)
+			return fmt.Errorf("wrong link %q, expected %q", hdr.Linkname, link)
 		}
 		return nil
 	}
@@ -1243,31 +1268,32 @@ func whiteoutEntry(name string) tarEntryValidator {
 
 	return func(hdr *tar.Header, b []byte) error {
 		if hdr.Typeflag != tar.TypeReg {
-			return errors.Errorf("not file type: %q", hdr.Typeflag)
+			return fmt.Errorf("not file type: %q", hdr.Typeflag)
 		}
 		if hdr.Name != whiteOut {
-			return errors.Errorf("wrong name %q, expected whiteout %q", hdr.Name, name)
+			return fmt.Errorf("wrong name %q, expected whiteout %q", hdr.Name, name)
 		}
 		return nil
 	}
 }
 
-func makeDiffTarTest(validators []tarEntryValidator, a, b fstest.Applier) func(*testing.T) {
-	return func(t *testing.T) {
-		ad, err := ioutil.TempDir("", "test-make-diff-tar-")
-		if err != nil {
-			t.Fatalf("failed to create temp dir: %v", err)
+func requireModTime(expected time.Time) tarEntryValidator {
+	return func(hdr *tar.Header, b []byte) error {
+		if !hdr.ModTime.Equal(expected) {
+			return fmt.Errorf("expected ModTime %v, got %v", expected, hdr.ModTime)
 		}
-		defer os.RemoveAll(ad)
+		return nil
+	}
+}
+
+func makeDiffTarTest(validators []tarEntryValidator, a, b fstest.Applier, opts ...WriteDiffOpt) func(*testing.T) {
+	return func(t *testing.T) {
+		ad := t.TempDir()
 		if err := a.Apply(ad); err != nil {
 			t.Fatalf("failed to apply a: %v", err)
 		}
 
-		bd, err := ioutil.TempDir("", "test-make-diff-tar-")
-		if err != nil {
-			t.Fatalf("failed to create temp dir: %v", err)
-		}
-		defer os.RemoveAll(bd)
+		bd := t.TempDir()
 		if err := fs.CopyDir(bd, ad); err != nil {
 			t.Fatalf("failed to copy dir: %v", err)
 		}
@@ -1275,7 +1301,7 @@ func makeDiffTarTest(validators []tarEntryValidator, a, b fstest.Applier) func(*
 			t.Fatalf("failed to apply b: %v", err)
 		}
 
-		rc := Diff(context.Background(), ad, bd)
+		rc := Diff(context.Background(), ad, bd, opts...)
 		defer rc.Close()
 
 		tr := tar.NewReader(rc)
@@ -1289,7 +1315,7 @@ func makeDiffTarTest(validators []tarEntryValidator, a, b fstest.Applier) func(*
 			}
 			var b []byte
 			if hdr.Typeflag == tar.TypeReg && hdr.Size > 0 {
-				b, err = ioutil.ReadAll(tr)
+				b, err = io.ReadAll(tr)
 				if err != nil {
 					t.Fatalf("tar read file error: %v", err)
 				}
@@ -1304,12 +1330,60 @@ func makeDiffTarTest(validators []tarEntryValidator, a, b fstest.Applier) func(*
 	}
 }
 
+func makeDiffTar(t *testing.T, a, b fstest.Applier, opts ...WriteDiffOpt) (digest.Digest, []byte) {
+	ad := t.TempDir()
+	if err := a.Apply(ad); err != nil {
+		t.Fatalf("failed to apply a: %v", err)
+	}
+
+	bd := t.TempDir()
+	if err := fs.CopyDir(bd, ad); err != nil {
+		t.Fatalf("failed to copy dir: %v", err)
+	}
+	if err := b.Apply(bd); err != nil {
+		t.Fatalf("failed to apply b: %v", err)
+	}
+
+	rc := Diff(context.Background(), ad, bd, opts...)
+	defer rc.Close()
+	var buf bytes.Buffer
+	r := io.TeeReader(rc, &buf)
+	dgst, err := digest.FromReader(r)
+	if err != nil {
+		t.Fatal(err)
+	}
+	if err = rc.Close(); err != nil {
+		t.Fatal(err)
+	}
+	return dgst, buf.Bytes()
+}
+
+func makeDiffTarReproTest(a, b fstest.Applier, opts ...WriteDiffOpt) func(*testing.T) {
+	return func(t *testing.T) {
+		const (
+			count = 30
+			delay = 100 * time.Millisecond
+		)
+		var lastDigest digest.Digest
+		for i := 0; i < count; i++ {
+			dgst, _ := makeDiffTar(t, a, b, opts...)
+			t.Logf("#%02d: %v: digest %s", i, time.Now(), dgst)
+			if lastDigest == "" {
+				lastDigest = dgst
+			} else if dgst != lastDigest {
+				t.Fatalf("expected digest %s, got %s", lastDigest, dgst)
+			}
+			time.Sleep(delay)
+		}
+	}
+}
+
 type diffApplier struct{}
 
 func (d diffApplier) TestContext(ctx context.Context) (context.Context, func(), error) {
-	base, err := ioutil.TempDir("", "test-diff-apply-")
+	base, err := os.MkdirTemp("", "test-diff-apply-")
 	if err != nil {
-		return ctx, nil, errors.Wrap(err, "failed to create temp dir")
+		return ctx, nil, fmt.Errorf("failed to create temp dir: %w", err)
 	}
 	return context.WithValue(ctx, d, base), func() {
 		os.RemoveAll(base)
@@ -1319,32 +1393,32 @@ func (d diffApplier) TestContext(ctx context.Context) (context.Context, func(),
 func (d diffApplier) Apply(ctx context.Context, a fstest.Applier) (string, func(), error) {
 	base := ctx.Value(d).(string)
 
-	applyCopy, err := ioutil.TempDir("", "test-diffapply-apply-copy-")
+	applyCopy, err := os.MkdirTemp("", "test-diffapply-apply-copy-")
 	if err != nil {
-		return "", nil, errors.Wrap(err, "failed to create temp dir")
+		return "", nil, fmt.Errorf("failed to create temp dir: %w", err)
 	}
 	defer os.RemoveAll(applyCopy)
 	if err = fs.CopyDir(applyCopy, base); err != nil {
-		return "", nil, errors.Wrap(err, "failed to copy base")
+		return "", nil, fmt.Errorf("failed to copy base: %w", err)
 	}
 	if err := a.Apply(applyCopy); err != nil {
-		return "", nil, errors.Wrap(err, "failed to apply changes to copy of base")
+		return "", nil, fmt.Errorf("failed to apply changes to copy of base: %w", err)
 	}
 
-	diffBytes, err := ioutil.ReadAll(Diff(ctx, base, applyCopy))
+	diffBytes, err := io.ReadAll(Diff(ctx, base, applyCopy))
 	if err != nil {
-		return "", nil, errors.Wrap(err, "failed to create diff")
+		return "", nil, fmt.Errorf("failed to create diff: %w", err)
 	}
 
 	if _, err = Apply(ctx, base, bytes.NewReader(diffBytes)); err != nil {
-		return "", nil, errors.Wrap(err, "failed to apply tar stream")
+		return "", nil, fmt.Errorf("failed to apply tar stream: %w", err)
 	}
 
 	return base, nil, nil
 }
 
 func readDirNames(p string) ([]string, error) {
-	fis, err := ioutil.ReadDir(p)
+	fis, err := os.ReadDir(p)
 	if err != nil {
 		return nil, err
 	}
