@@ -1,5 +1,4 @@
 //go:build linux
-// +build linux
 
 /*
    Copyright The containerd Authors.
@@ -24,11 +23,13 @@ import (
 	"fmt"
 	"sync"
 
-	"github.com/containerd/containerd/log"
+	cmetrics "github.com/containerd/containerd/metrics"
 	"github.com/containerd/containerd/metrics/cgroups/common"
 	v2 "github.com/containerd/containerd/metrics/types/v2"
 	"github.com/containerd/containerd/namespaces"
-	"github.com/containerd/typeurl"
+	"github.com/containerd/containerd/pkg/timeout"
+	"github.com/containerd/log"
+	"github.com/containerd/typeurl/v2"
 	"github.com/docker/go-metrics"
 	"github.com/prometheus/client_golang/prometheus"
 )
@@ -129,23 +130,23 @@ func (c *Collector) collect(entry entry, ch chan<- prometheus.Metric, block bool
 	if wg != nil {
 		defer wg.Done()
 	}
+
 	t := entry.task
-	ctx := namespaces.WithNamespace(context.Background(), t.Namespace())
-	stats, err := t.Stats(ctx)
+	ctx, cancel := timeout.WithContext(context.Background(), cmetrics.ShimStatsRequestTimeout)
+	stats, err := t.Stats(namespaces.WithNamespace(ctx, t.Namespace()))
+	cancel()
+
 	if err != nil {
 		log.L.WithError(err).Errorf("stat task %s", t.ID())
 		return
 	}
-	data, err := typeurl.UnmarshalAny(stats)
-	if err != nil {
+
+	s := &v2.Metrics{}
+	if err := typeurl.UnmarshalTo(stats, s); err != nil {
 		log.L.WithError(err).Errorf("unmarshal stats for %s", t.ID())
 		return
 	}
-	s, ok := data.(*v2.Metrics)
-	if !ok {
-		log.L.WithError(err).Errorf("invalid metric type for %s", t.ID())
-		return
-	}
+
 	ns := entry.ns
 	if ns == nil {
 		ns = c.ns
