@@ -19,25 +19,31 @@ package server
 import (
 	"context"
 	"os"
+	goruntime "runtime"
 	"strings"
 	"testing"
 	"time"
 
+	"github.com/containerd/containerd/containers"
 	"github.com/containerd/containerd/errdefs"
 	"github.com/containerd/containerd/oci"
 	criconfig "github.com/containerd/containerd/pkg/cri/config"
 	containerstore "github.com/containerd/containerd/pkg/cri/store/container"
 	imagestore "github.com/containerd/containerd/pkg/cri/store/image"
 	"github.com/containerd/containerd/plugin"
-	"github.com/containerd/containerd/reference/docker"
+	"github.com/containerd/containerd/protobuf/types"
 	"github.com/containerd/containerd/runtime/linux/runctypes"
 	runcoptions "github.com/containerd/containerd/runtime/v2/runc/options"
+	"github.com/containerd/typeurl/v2"
+	docker "github.com/distribution/reference"
 
 	imagedigest "github.com/opencontainers/go-digest"
 	runtimespec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/pelletier/go-toml"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+
+	runtime "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
 // TestGetUserFromImage tests the logic of getting image uid or user name of image user.
@@ -72,10 +78,11 @@ func TestGetUserFromImage(t *testing.T) {
 			name: "test",
 		},
 	} {
-		t.Logf("TestCase - %q", c)
-		actualUID, actualName := getUserFromImage(test.user)
-		assert.Equal(t, test.uid, actualUID)
-		assert.Equal(t, test.name, actualName)
+		t.Run(c, func(t *testing.T) {
+			actualUID, actualName := getUserFromImage(test.user)
+			assert.Equal(t, test.uid, actualUID)
+			assert.Equal(t, test.name, actualName)
+		})
 	}
 }
 
@@ -109,12 +116,13 @@ func TestGetRepoDigestAndTag(t *testing.T) {
 			expectedRepoTag:    "",
 		},
 	} {
-		t.Logf("TestCase %q", desc)
-		named, err := docker.ParseDockerRef(test.ref)
-		assert.NoError(t, err)
-		repoDigest, repoTag := getRepoDigestAndTag(named, digest, test.schema1)
-		assert.Equal(t, test.expectedRepoDigest, repoDigest)
-		assert.Equal(t, test.expectedRepoTag, repoTag)
+		t.Run(desc, func(t *testing.T) {
+			named, err := docker.ParseDockerRef(test.ref)
+			assert.NoError(t, err)
+			repoDigest, repoTag := getRepoDigestAndTag(named, digest, test.schema1)
+			assert.Equal(t, test.expectedRepoDigest, repoDigest)
+			assert.Equal(t, test.expectedRepoTag, repoTag)
+		})
 	}
 }
 
@@ -361,17 +369,18 @@ func TestEnvDeduplication(t *testing.T) {
 			},
 		},
 	} {
-		t.Logf("TestCase %q", desc)
-		var spec runtimespec.Spec
-		if len(test.existing) > 0 {
-			spec.Process = &runtimespec.Process{
-				Env: test.existing,
+		t.Run(desc, func(t *testing.T) {
+			var spec runtimespec.Spec
+			if len(test.existing) > 0 {
+				spec.Process = &runtimespec.Process{
+					Env: test.existing,
+				}
 			}
-		}
-		for _, kv := range test.kv {
-			oci.WithEnv([]string{kv[0] + "=" + kv[1]})(context.Background(), nil, nil, &spec)
-		}
-		assert.Equal(t, test.expected, spec.Process.Env)
+			for _, kv := range test.kv {
+				oci.WithEnv([]string{kv[0] + "=" + kv[1]})(context.Background(), nil, nil, &spec)
+			}
+			assert.Equal(t, test.expected, spec.Process.Env)
+		})
 	}
 }
 
@@ -599,3 +608,59 @@ func TestValidateTargetContainer(t *testing.T) {
 	}
 
 }
+
+func TestGetRuntimeOptions(t *testing.T) {
+	_, err := getRuntimeOptions(containers.Container{})
+	require.NoError(t, err)
+
+	var pbany *types.Any               // This is nil.
+	var typeurlAny typeurl.Any = pbany // This is typed nil.
+	_, err = getRuntimeOptions(containers.Container{Runtime: containers.RuntimeInfo{Options: typeurlAny}})
+	require.NoError(t, err)
+}
+
+func TestHostNetwork(t *testing.T) {
+	tests := []struct {
+		name     string
+		c        *runtime.PodSandboxConfig
+		expected bool
+	}{
+		{
+			name: "when pod namespace return false",
+			c: &runtime.PodSandboxConfig{
+				Linux: &runtime.LinuxPodSandboxConfig{
+					SecurityContext: &runtime.LinuxSandboxSecurityContext{
+						NamespaceOptions: &runtime.NamespaceOption{
+							Network: runtime.NamespaceMode_POD,
+						},
+					},
+				},
+			},
+			expected: false,
+		},
+		{
+			name: "when node namespace return true",
+			c: &runtime.PodSandboxConfig{
+				Linux: &runtime.LinuxPodSandboxConfig{
+					SecurityContext: &runtime.LinuxSandboxSecurityContext{
+						NamespaceOptions: &runtime.NamespaceOption{
+							Network: runtime.NamespaceMode_NODE,
+						},
+					},
+				},
+			},
+			expected: true,
+		},
+	}
+
+	for _, tt := range tests {
+		if goruntime.GOOS != "linux" {
+			t.Skip()
+		}
+		t.Run(tt.name, func(t *testing.T) {
+			if hostNetwork(tt.c) != tt.expected {
+				t.Errorf("failed hostNetwork got %t expected %t", hostNetwork(tt.c), tt.expected)
+			}
+		})
+	}
+}
