@@ -1,4 +1,4 @@
-// +build linux
+//go:build linux
 
 /*
    Copyright The containerd Authors.
@@ -19,10 +19,12 @@
 package plugin
 
 import (
-	"github.com/containerd/containerd/platforms"
+	"errors"
+	"fmt"
+
 	"github.com/containerd/containerd/plugin"
 	"github.com/containerd/containerd/snapshots/devmapper"
-	"github.com/pkg/errors"
+	"github.com/containerd/platforms"
 )
 
 func init() {
@@ -39,13 +41,14 @@ func init() {
 			}
 
 			if config.PoolName == "" {
-				return nil, errors.New("devmapper not configured")
+				return nil, fmt.Errorf("devmapper not configured: %w", plugin.ErrSkipPlugin)
 			}
 
 			if config.RootPath == "" {
 				config.RootPath = ic.Root
 			}
 
+			ic.Meta.Exports[plugin.SnapshotterRootDir] = config.RootPath
 			return devmapper.NewSnapshotter(ic.Context, config)
 		},
 	})
